// 
// Copyright (C) 2014, NorthTrader
// V1  Sep-2014  NT  Initial version
//

#region Using declarations
using System;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// TZBarAnalysis (analysis of Transient Zones (TZs), which are zones that price doesn't hit within h bars left and right of middle bar)
    /// </summary>
    [Description("This version (V1) plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) " +
		"on the chart and provides bar analysis statistics.\n" + 
		"\"k=2/14\" : avg height of TZ is 2 pips, max is 14 pips\n" +
		"\"consec\" : % of consecutive MidTZ bars\n" + 
		"\"FullR\" : % of bars that are recurrent to the left and right of middle bar\n" +
		"\"2/1\" (on chart TZ bar) : left-TZ is 2 pips high, right-TZ is 1 pip\n" +
		"\"-/-\" (on chart bar) : TZ bar is ignored due to kMin setting\n" +
		"\"x\" (above chart bar) : bar contains prices that are right-only-transient\n"
	)]
    [Gui.Design.DisplayName("TZBarAnalysis")]
    public class TZBarAnalysis : Indicator
    {
        #region Variables

		//Parameters 
		private int iH_Left = 13;     //hLeft
		private int iH_Right = 13;    //hRight
		private int iK_Min = 1;       //kMin

		//Plot colours 
		private Color colTipTZ = Color.Blue;
		private Color colMidTZ = Color.DeepPink;
		private Color colBarStd = Color.Blue;
		private Color colBarIgnore = Color.Red;
		private Color colBarText = Color.Empty;

		//Other 
		private	int iHRange, iBarMid, iBarFirst, iBarLast;
		private	int iTotalBars, iConsecMidTZBars, iLastMidTZBar;
		private	int iTipTZBars, iTipMinK, iTipMaxK, iTipTotalK;
		private	int iMidTZBars, iMidMinK, iMidMaxK, iMidTotalK;
		private	int iLeftTZBars, iRightTZBars, iFullRecurrentBars, iBelowMinTZBars;
		private	string sBarText;
		private double dMinK, dTextOffset;
		
        #endregion

        protected override void Initialize()
        {
			//Initialize variables
			iHRange = iH_Left + iH_Right + 1;
			iBarMid = iH_Right;
			iBarFirst = iBarMid + iH_Left;
			iBarLast = 0;  //iBarMid - iH_Left;
			iTotalBars = iConsecMidTZBars = iLastMidTZBar = 0;
			iTipTZBars = iTipMaxK = iTipTotalK = 0;
			iMidTZBars = iMidMaxK = iMidTotalK = 0;
			iLeftTZBars = iRightTZBars = iFullRecurrentBars = iBelowMinTZBars = 0;
			iTipMinK = iMidMinK = 9999;

			DisplayInDataBox	= false;
			PaintPriceMarkers	= false;
            Overlay				= true;
			CalculateOnBarClose = true;	//not updated on every tick

        }

		protected override void OnBarUpdate()
        {
			if (CurrentBar == 0) SetVars();
			if (CurrentBar < iHRange) return;
			
			double ldLeftStart = 0;
			double ldTextPos = High[iBarMid] + dTextOffset;
			int liLeftK = 0;

			iTotalBars++;
			sBarText = "";
			
			//Calculate range of previous H bars
			double ldLeftLow = Low[iBarMid + 1];
			double ldLeftHigh = High[iBarMid + 1];
			for (int liBar = iBarMid + 2; liBar <= iBarFirst; liBar++)
			{	
				if (Low[liBar] < ldLeftLow) ldLeftLow = Low[liBar]; 
				if (High[liBar] > ldLeftHigh) ldLeftHigh = High[liBar]; 
			}
			//Calculate range of next H bars
			double ldRightLow = Low[iBarLast];
			double ldRightHigh = High[iBarLast];
			for (int liBar = iBarLast + 1; liBar < iBarMid; liBar++)
			{	
				if (Low[liBar] < ldRightLow) ldRightLow = Low[liBar]; 
				if (High[liBar] > ldRightHigh) ldRightHigh = High[liBar]; 
			}

			//Derive bar type
			if (ldLeftLow > Low[iBarMid] && ldRightLow > Low[iBarMid])
			{
				//TIP Bottom TZ bar
				ProcessTZBar(true, ldLeftLow, Low[iBarMid], ldRightLow, Low[iBarMid]);
				ldTextPos = Low[iBarMid] - dTextOffset;
			}
			else if (ldLeftHigh < High[iBarMid] && ldRightHigh < High[iBarMid])
			{
				//TIP Top TZ bar
				ProcessTZBar(true, High[iBarMid], ldLeftHigh, High[iBarMid], ldRightHigh);
			}
			else if (ldLeftLow > (ldRightHigh + TickSize))
			{
				//MID Down TZ bar
				ldRightHigh+= TickSize;
				ProcessTZBar(false, ldLeftLow, ldRightHigh, ldLeftLow, ldRightHigh);
			}
			else if (ldLeftHigh < (ldRightLow - TickSize))
			{
				//MID Up TZ bar
				ldRightLow-= TickSize;
				ProcessTZBar(false, ldRightLow, ldLeftHigh, ldRightLow, ldLeftHigh);
			}
			else if (ldRightLow <= Low[iBarMid] && ldRightHigh >= High[iBarMid])
			{
				//Right-side recurrent, so check left-side 
				if (ldLeftLow > Low[iBarMid] || ldLeftHigh < High[iBarMid])
				{
					//Left-TZ-only bar
					iLeftTZBars++;
				}
				else 
				{
					//Fully recurrent (left and right) bar
					iFullRecurrentBars++;
				}
			}
			else
			{
				//Right-TZ-only bar
				colBarText = Color.Blue;
				sBarText = "x";
				iRightTZBars++;
			}
			
			//Annotate bar
			if (sBarText != "") DrawText("A" + CurrentBar, sBarText, iBarMid, ldTextPos, colBarText);
			
			//Process and display stats on last bar
			if (CurrentBar == Count - 2) ProcessStats();		
		
		}

        #region Functions

		private void SetVars()
		{
			//Called on first bar of chart only due to some properties not available during Initialise()
			dMinK = iK_Min * TickSize;  //Price range of minimum k
			int liTextOffset = 1;  //No of pips away from bar high or low to display text
			if (BarsPeriod.Id == PeriodType.Day) liTextOffset = 10;
			else if (BarsPeriod.Id == PeriodType.Week) liTextOffset = 15;
			else if (BarsPeriod.Id == PeriodType.Month) liTextOffset = 20;
			else
			{
				//Minute bars, Tick bars and others 
				if (BarsPeriod.Value < 5) liTextOffset = 1;
				else if (BarsPeriod.Value < 15) liTextOffset = 2;
				else if (BarsPeriod.Value < 60) liTextOffset = 3;
				else if (BarsPeriod.Value < 240) liTextOffset = 4;
				else liTextOffset = 8;
			}
			dTextOffset = TickSize * liTextOffset;
		}

		private void ProcessTZBar(bool abTip, double adLeftHighPrice, double adLeftLowPrice,
			double adRightHighPrice, double adRightLowPrice)
		{
			//Called when bar has potential TZ
			//
			int liLeftK = (int)Math.Round((adLeftHighPrice - adLeftLowPrice) / TickSize, 0);
			int liRightK = (int)Math.Round((adRightHighPrice - adRightLowPrice) / TickSize, 0);
			int liK = Math.Min(liLeftK, liRightK);
			Color lcolTZ = Color.Empty;
			if (liK < iK_Min)
			{
				colBarText = colBarIgnore;
				sBarText = "-/-";
				iBelowMinTZBars++;
				return;
			}
			else colBarText = colBarStd;
			if (abTip)
			{
				iTipTZBars++;
				sBarText = liLeftK.ToString() + "/" + liRightK.ToString();
				lcolTZ = colTipTZ;
				iTipTotalK += liK;
				if (liK < iTipMinK) iTipMinK = liK;
				if (liK > iTipMaxK) iTipMaxK = liK;
			}
			else
			{
				iMidTZBars++;
				sBarText = liK.ToString();
				lcolTZ = colMidTZ;
				iMidTotalK += liK;
				if (liK < iMidMinK) iMidMinK = liK;
				if (liK > iMidMaxK) iMidMaxK = liK;
				//Check for consecutive Mid TZ bars
				if (CurrentBar == (iLastMidTZBar + 1)) iConsecMidTZBars++;
				iLastMidTZBar = CurrentBar;  //Actually not TZ bar, but OK for comparison
			}
			//Mark TZ on chart
			DrawRectangle("LTZ" + CurrentBar, true, iBarFirst, adLeftHighPrice, iBarMid + 1, adLeftLowPrice, Color.Empty, lcolTZ, 4);
			DrawRectangle("RTZ" + CurrentBar, true, iBarMid - 1, adRightHighPrice, iBarLast, adRightLowPrice, Color.Empty, lcolTZ, 4);
		}

		private void ProcessStats()
		{
			//Called on last bar
			string lsSep = ", ";
			double ldTipTZPercent = (double)iTipTZBars / iTotalBars;
			double ldMidTZPercent = (double)iMidTZBars / iTotalBars;
			double ldLeftTZPercent = (double)iLeftTZBars / iTotalBars;
			double ldRightTZPercent = (double)iRightTZBars / iTotalBars;
			double ldFullRecurrentPercent = (double)iFullRecurrentBars / iTotalBars;
			double ldFullAndRightRecurrentPercent = (double)(iFullRecurrentBars + iLeftTZBars) / iTotalBars;
			double ldBelowMinTZPercent = (double)iBelowMinTZBars / iTotalBars;
			double ldTipTZAvgK = (double)iTipTotalK / iTipTZBars;
			double ldMidTZAvgK = (double)iMidTotalK / iMidTZBars;
			double ldConsecMidTZBarsPercent = (double)iConsecMidTZBars / iMidTZBars;
			
			string lsStats = "h=" + iH_Left + "/" + iH_Right + lsSep;
			lsStats += "kMin=" + iK_Min + lsSep;
			lsStats += "Pip=" + TickSize.ToString() + lsSep;
			lsStats += "Bars=" + iTotalBars + lsSep;
			lsStats += "TipTZs=" + ldTipTZPercent.ToString("0.0%") + 
				" (k=" + ldTipTZAvgK.ToString("0") + "/" + iTipMaxK + ")" + lsSep;
			lsStats += "MidTZs=" + ldMidTZPercent.ToString("0.0%") +
				" (k=" + ldMidTZAvgK.ToString("0") + "/" + iMidMaxK + ", consec=" +
				ldConsecMidTZBarsPercent.ToString("0.0%") + ")" + lsSep;
			if (ldBelowMinTZPercent > 0) lsStats += "BelowMinTZs=" + ldBelowMinTZPercent.ToString("0.0%") + lsSep;
			lsStats += "Full&RightR=" + ldFullAndRightRecurrentPercent.ToString("0.0%") + lsSep;
			lsStats += "LeftOnlyR=" + ldRightTZPercent.ToString("0.0%") + lsSep;
			lsStats += "RightOnlyR=" + ldLeftTZPercent.ToString("0.0%") + lsSep;
			lsStats += "FullR=" + ldFullRecurrentPercent.ToString("0.0%");
			DrawTextFixed("stats", lsStats, TextPosition.TopRight);

/*
			ClearOutputWindow();
			Print("h Left = " + iH_Left + lsSep + "h Right = " + iH_Right + lsSep + "k Min = " + iK_Min);
			Print("Total bars = " + iTotalBars);
*/
		}
		
		#endregion

        #region Properties
		
		[Description("Number of bars to the left of middle bar.")]
        [GridCategory("Parameters")]
        public int hLeft
        {
            get { return iH_Left; }
            set { iH_Left = Math.Max(1, value); }
        }
		[Description("Number of bars to the right of middle bar.")]
        [GridCategory("Parameters")]
        public int hRight
        {
            get { return iH_Right; }
            set { iH_Right = Math.Max(1, value); }
        }
		[Description("Minimum height of TZ (in pips).")]
        [GridCategory("Parameters")]
        public int kMin
        {
            get { return iK_Min; }
            set { iK_Min = Math.Max(1, value); }
        }
		
		[Description("Color of Tip TZ")]
        [GridCategory("Plots")]
        public Color TZTipColor
        {
            get { return colTipTZ; }
            set { colTipTZ = value; }
        }
		[Description("Color of Mid TZ")]
        [GridCategory("Plots")]
        public Color TZMidColor
        {
            get { return colMidTZ; }
            set { colMidTZ = value; }
        }
		[Description("Color of Standard Bar text")]
        [GridCategory("Plots")]
        public Color BarStdColor
        {
            get { return colBarStd; }
            set { colBarStd = value; }
        }
		[Description("Color of Ignored Bar text")]
        [GridCategory("Plots")]
        public Color BarIgnoreColor
        {
            get { return colBarIgnore; }
            set { colBarIgnore = value; }
        }

		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TZBarAnalysis[] cacheTZBarAnalysis = null;

        private static TZBarAnalysis checkTZBarAnalysis = new TZBarAnalysis();

        /// <summary>
        /// This indicator plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides bar analysis.
        /// </summary>
        /// <returns></returns>
        public TZBarAnalysis TZBarAnalysis(Color barIgnoreColor, Color barStdColor, int hLeft, int hRight, int kMin, Color tZMidColor, Color tZTipColor)
        {
            return TZBarAnalysis(Input, barIgnoreColor, barStdColor, hLeft, hRight, kMin, tZMidColor, tZTipColor);
        }

        /// <summary>
        /// This indicator plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides bar analysis.
        /// </summary>
        /// <returns></returns>
        public TZBarAnalysis TZBarAnalysis(Data.IDataSeries input, Color barIgnoreColor, Color barStdColor, int hLeft, int hRight, int kMin, Color tZMidColor, Color tZTipColor)
        {
            if (cacheTZBarAnalysis != null)
                for (int idx = 0; idx < cacheTZBarAnalysis.Length; idx++)
                    if (cacheTZBarAnalysis[idx].BarIgnoreColor == barIgnoreColor && cacheTZBarAnalysis[idx].BarStdColor == barStdColor && cacheTZBarAnalysis[idx].hLeft == hLeft && cacheTZBarAnalysis[idx].hRight == hRight && cacheTZBarAnalysis[idx].kMin == kMin && cacheTZBarAnalysis[idx].TZMidColor == tZMidColor && cacheTZBarAnalysis[idx].TZTipColor == tZTipColor && cacheTZBarAnalysis[idx].EqualsInput(input))
                        return cacheTZBarAnalysis[idx];

            lock (checkTZBarAnalysis)
            {
                checkTZBarAnalysis.BarIgnoreColor = barIgnoreColor;
                barIgnoreColor = checkTZBarAnalysis.BarIgnoreColor;
                checkTZBarAnalysis.BarStdColor = barStdColor;
                barStdColor = checkTZBarAnalysis.BarStdColor;
                checkTZBarAnalysis.hLeft = hLeft;
                hLeft = checkTZBarAnalysis.hLeft;
                checkTZBarAnalysis.hRight = hRight;
                hRight = checkTZBarAnalysis.hRight;
                checkTZBarAnalysis.kMin = kMin;
                kMin = checkTZBarAnalysis.kMin;
                checkTZBarAnalysis.TZMidColor = tZMidColor;
                tZMidColor = checkTZBarAnalysis.TZMidColor;
                checkTZBarAnalysis.TZTipColor = tZTipColor;
                tZTipColor = checkTZBarAnalysis.TZTipColor;

                if (cacheTZBarAnalysis != null)
                    for (int idx = 0; idx < cacheTZBarAnalysis.Length; idx++)
                        if (cacheTZBarAnalysis[idx].BarIgnoreColor == barIgnoreColor && cacheTZBarAnalysis[idx].BarStdColor == barStdColor && cacheTZBarAnalysis[idx].hLeft == hLeft && cacheTZBarAnalysis[idx].hRight == hRight && cacheTZBarAnalysis[idx].kMin == kMin && cacheTZBarAnalysis[idx].TZMidColor == tZMidColor && cacheTZBarAnalysis[idx].TZTipColor == tZTipColor && cacheTZBarAnalysis[idx].EqualsInput(input))
                            return cacheTZBarAnalysis[idx];

                TZBarAnalysis indicator = new TZBarAnalysis();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.BarIgnoreColor = barIgnoreColor;
                indicator.BarStdColor = barStdColor;
                indicator.hLeft = hLeft;
                indicator.hRight = hRight;
                indicator.kMin = kMin;
                indicator.TZMidColor = tZMidColor;
                indicator.TZTipColor = tZTipColor;
                Indicators.Add(indicator);
                indicator.SetUp();

                TZBarAnalysis[] tmp = new TZBarAnalysis[cacheTZBarAnalysis == null ? 1 : cacheTZBarAnalysis.Length + 1];
                if (cacheTZBarAnalysis != null)
                    cacheTZBarAnalysis.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheTZBarAnalysis = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// This indicator plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides bar analysis.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TZBarAnalysis TZBarAnalysis(Color barIgnoreColor, Color barStdColor, int hLeft, int hRight, int kMin, Color tZMidColor, Color tZTipColor)
        {
            return _indicator.TZBarAnalysis(Input, barIgnoreColor, barStdColor, hLeft, hRight, kMin, tZMidColor, tZTipColor);
        }

        /// <summary>
        /// This indicator plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides bar analysis.
        /// </summary>
        /// <returns></returns>
        public Indicator.TZBarAnalysis TZBarAnalysis(Data.IDataSeries input, Color barIgnoreColor, Color barStdColor, int hLeft, int hRight, int kMin, Color tZMidColor, Color tZTipColor)
        {
            return _indicator.TZBarAnalysis(input, barIgnoreColor, barStdColor, hLeft, hRight, kMin, tZMidColor, tZTipColor);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// This indicator plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides bar analysis.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TZBarAnalysis TZBarAnalysis(Color barIgnoreColor, Color barStdColor, int hLeft, int hRight, int kMin, Color tZMidColor, Color tZTipColor)
        {
            return _indicator.TZBarAnalysis(Input, barIgnoreColor, barStdColor, hLeft, hRight, kMin, tZMidColor, tZTipColor);
        }

        /// <summary>
        /// This indicator plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) on the chart and provides bar analysis.
        /// </summary>
        /// <returns></returns>
        public Indicator.TZBarAnalysis TZBarAnalysis(Data.IDataSeries input, Color barIgnoreColor, Color barStdColor, int hLeft, int hRight, int kMin, Color tZMidColor, Color tZTipColor)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TZBarAnalysis(input, barIgnoreColor, barStdColor, hLeft, hRight, kMin, tZMidColor, tZTipColor);
        }
    }
}
#endregion
